/****************************************************************************

Author : Renato Ferrero
Version : 1.0
Date : 23/02/2013

Revisioned: Mauro Laurenti
Date : 29/04/2013

CopyRight 2013 all rights are reserved


********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by the Author is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

****************************************************************************/

/****************************************************************************

DESCRIZIONE
Il ricevitore qui descritto funziona indipendentemente dal protocollo utilizzato
in quanto non rileva il codice composto dai vari bit ma misura, ed in seguito
confronta o memorizza, la durata di ogni singolo stato del  segnale rilevato.
Ne deriva quindi che ogni bit trasmesso,composto da un impulso con uno stato
basso ed uno alto, viene suddiviso in due nuove variabili denominate nel
firmware Dsi  (Durata Stato Impulso), aventi un valore  proporzionale alla
durata dello stato del segnale rilevato. Quindi, ad esempio, per misurare un
pacchetto dimpulsi a 20 bit occorrono 40 variabili Dsi .Ogni singolo canale del
ricevitore  progettato per ricevere ed analizzare comandi composti da 60 Dsi e
quindi da un massimo di 30 bit. Nel caso in cui venga rilevato un pacchetto
inviato con protocollo RC5 della Philips in cui il terzo bit varia ad ogni invio
di dati, al momento del confronto verranno analizzati solo Dsi dal 6 in poi.

Nota:
 
PROVATO CON TELECOMANDI
SONY, SAMSUG, UNITED, PHILIPS, RGB, LG

****************************************************************************/

#include <xc.h>

#include <portb.h>

#include <pwm.h>
#include <timers.h>

#include "intEEPROM.h"
#include "intEEPROM.c"

#include "delay.h"
#include "delay.c"

#include "PIC18F2331_config.h"

//************************************************************
// Dichiarazione Prototipi di funzione
//************************************************************

unsigned char ControllaSeEunaSequenza(void);
unsigned char ConfrontaSequenza(void);
unsigned char ContaPremiPulsante(void);

void ApprendiSequenza(void);
void RilevaSequenzaIR(void);
void GestioneUscitaPWM (void);
void GestioneUscitaTRIAC (void);
void RegolaLuminositaPWM (void);
void GestioneTastoPremuto (void);
void RegolaLuminositaTriac(void);
void TestUscite (void);


//************************************************************
// Dichiarazione Costanti
//************************************************************

// INPUT - OUTPUT
#define JUMPER_CAN_2 PORTBbits.RB0
#define JUMPER_CAN_3 PORTBbits.RB1
#define PUNTO_ZERO PORTCbits.RC3
#define TASTO_APPRENDI PORTBbits.RB4
#define STATO_INPUT_IR PORTBbits.RB5
#define USCITA_CANALE_2 LATAbits.LATA0
#define USCITA_CANALE_3 LATAbits.LATA1
#define OUT_TRIAC LATAbits.LATA3
#define ZERO_CROSS  LATAbits.LATA4
#define LED_ROSSO LATCbits.LATC5
#define LED_GIALLO LATCbits.LATC4

// COSTANTI
#define NUM_MAX_DSI 60
#define MINIMO_TRIAC 65
#define DELTA 6
#define NUM_MAX_CONTR_IR 200
#define DURATA_MIN_PRIMO_DSIi 60

#define TASTO_1 1
#define TASTO_2 2
#define TASTO_3 3
#define TASTO_4 4

// serve per riprendere lo sfasamento provocato dall'alimentazione RC
// del fotoaccoppiatore per il punto zero
#define SFASAMENTO_RETE_RC 27
                            
#define T_MAX_ATTESA 190000
#define PRIMO_DSI_PHILIPS 70


//************************************************************
// Dichiarazione Variabili
//************************************************************

unsigned char ContDsiRic[NUM_MAX_DSI];

int x;
int x2;
int ConteggioDsi;
int TimMaxIntervalloImpulsiIR = 7500;
unsigned int ContSleep =0;
unsigned int ContTimer =0;
unsigned char ContSfasam =0;
unsigned char FlagTastoApprendi = 0;
unsigned char FlagArrivoIR =0;
unsigned char StatoUscitaPWM=0 ;
unsigned char StatoPulsante=0 ;
unsigned char NumVoltePremApprendi =0;
unsigned char NumTastoPremuto =0;
unsigned char FlagPuntoZero =0;
unsigned char FlagSegnalePhilips =0;

// variabili per controllo luminosit
// Periodo del segnale PWM
unsigned char period;   
int duty_cycle = 0;     
int PercLumin = 1023;
unsigned char FlagAumentaLuminosita = 0;
unsigned char FlagRegolPwmEffettuata =0;

// variabili per controllo luminosit TRIAC
unsigned char MemoriaLuminTriac = 30;
unsigned char AttivaTriac =30;
unsigned char FlagAumLuminTriac = 0;
unsigned char StatoUscitaTRIAC =0;
unsigned char FlagRegolTriacOk = 0;
unsigned char StatoUscitaTriac =0;


//************************************************************
// Impostazione e Gestione priorita' alta 
//************************************************************

__interrupt (high_priority) void ISR_cnc_signals (void) {


    // Controllo che l'interrupt sia stato generato da PORTB
    if ((INTCONbits.RBIF == 1 )&&(FlagTastoApprendi == 0)) {

        if (PORTBbits.RB5 == 0){
            INTCONbits.RBIE = 0;
            FlagArrivoIR = 1;
        } else {

            //pausa filtraggio spike
            delay_ms (5);

            // Controllo la pressione di RB4...tasto apprendi
            if (PORTBbits.RB4 == 0) {
                INTCONbits.RBIE = 0;
                FlagTastoApprendi =1;
            }
        }
    }

    // Resetto il flag d'interrupt per permettere nuove interruzioni
    INTCONbits.RBIF = 0;
}

//************************************************************
// Impostazione e Gestione priorita' bassa 
//************************************************************

__interrupt (low_priority) void ISR_cnc_measures (void) {

    // Controllo che l'interrupt sia stato generato da Timer0
    if (INTCONbits.TMR0IF == 1 ) {
        ContSleep ++ ;
        ContTimer ++ ;
        ZERO_CROSS =0;
        ConteggioDsi ++;

        if(PUNTO_ZERO == 1){
            FlagPuntoZero =1;
        }

        if ((PUNTO_ZERO ==0)&&(FlagPuntoZero ==1)){
            ContSfasam ++;
        }

        if (ContSfasam > SFASAMENTO_RETE_RC){
            ZERO_CROSS = 1;
            ContSfasam =0;
            OUT_TRIAC =0;
            ContTimer =0;
            FlagPuntoZero =0;
        }

        if ((ContTimer >AttivaTriac) && (StatoUscitaTriac ==1)){
            OUT_TRIAC =1;
        }
    }

    // Resetto il flag d'interrupt per permettere nuove interruzioni
    INTCONbits.TMR0IF = 0;
}


//************************************************************
//*****************   INIZIO PROGRAMMA PRINCIPALE
//************************************************************

void main (void){
	
    // Imposto PORTA tutte uscite
    LATA = 0x00;
    TRISA = 0x00;

    // Imposto PORTB tutti ingressi
    LATB = 0x00;
    TRISB = 0xFF;

    // Imposto PORTC tutti ingressi ed RC1-4-5 come uscita
    LATC = 0x00;
    TRISC = 0b11001101;

    // DETERMINA FREQUENZA CLOCK INTERNO A 8MHZ
    OSCCON = 0x72;

    LED_GIALLO = 1;

    // Apro il TMR2 per il PWM
    OpenTimer2( TIMER_INT_OFF & T2_PS_1_1 & T2_POST_1_1);
    // Imposto una frequenza di 20KHz  (con quarzo da 20MHZ)
    //                          8KHz  (con oscillatore interno 8MHz)
    period = 249;
    // Apro il moldulo PWM
    OpenPWM2( period );
    SetDCPWM2 (0);
    StatoUscitaPWM =0;

    //************************************************************
    // Abilito i pulsanti per le interruzioni ad alta priorita'
    //************************************************************

    // Abilita i resistori di pull-up sulla PORTB
    EnablePullups();
    // Abilito le interruzioni su PORTB
    INTCONbits.RBIE = 1;
    // Abilito le interruzioni su PORTB come alta priorita'
    INTCON2bits.RBIP = 1;

    //************************************************************
    // Abilito il Timer0 per funzionare a bassa priorit
    //************************************************************
    // Modalit a 8 bit
    T0CONbits.T016BIT = 1;//  0= 16 bit
    // Clock interno
    T0CONbits.T0CS = 0;
    // Disabilito Prescaler
    T0CONbits.PSA = 1;
    // Prescaler 02
    T0CONbits.T0PS0 = 0;
    T0CONbits.T0PS1 = 0;
    T0CONbits.T0PS2 = 0;
    // Abilito le interruzioni del Timer0
    INTCONbits.TMR0IE = 1;
    // Abilito le interruzioni del Timer0 come bassa priorit
    INTCON2bits.TMR0IP = 0;
    // Abilito il Timer0
    T0CONbits.TMR0ON = 1;


    //************************************************************
    // Abilito le interruzioni
    //************************************************************

    // Abilito modalita' interruzione a due livelli alta e bassa
    RCONbits.IPEN = 1;

    // Abilito gli interrupt a bassa priorita'
    INTCONbits.GIEL = 1;

    // Abilito gli interrupt ad alta priorita'
    INTCONbits.GIEH = 1;

    //************************************************************
    // Inizializzazione array
    //************************************************************
    for (x2=0;x2<NUM_MAX_DSI;x2++){
        ContDsiRic[x2]=0;
    }


    //************************************************************
    // Test Uscite
    //************************************************************

    if(TASTO_APPRENDI == 0){
        TestUscite();
        FlagTastoApprendi=0;
        INTCONbits.RBIE = 1;
    }

//************************************************************************
//***************            CICLO INFINITO                     **********
//************************************************************************

    while (1){

        //***** ESEGUITO SE E' PREMUTO IL TASTO APPRENDI
        if(FlagTastoApprendi==1){  
            ContaPremiPulsante();
            ApprendiSequenza();
            FlagTastoApprendi=0;
            INTCONbits.RBIE = 1;
        }

        // ***** ESEGUITO SE E' IN ARRIVO UNA SEQUENZA
        if(FlagArrivoIR){   
            LED_ROSSO = 1 ;

        if(ControllaSeEunaSequenza()){
            NumTastoPremuto = ConfrontaSequenza();

            switch (NumTastoPremuto){
                //********************* E' PREMUTO IL TASTO MEMORIZZATO 1
                case TASTO_1:
                        GestioneUscitaPWM();
                        break;

                //********************* E' PREMUTO IL TASTO MEMORIZZATO 2
                case TASTO_2:
                        if (JUMPER_CAN_2 ==0){
                            USCITA_CANALE_2 =1;
                        }else{
                            USCITA_CANALE_2 = !(USCITA_CANALE_2);
                        }

                        GestioneTastoPremuto();
                        if (JUMPER_CAN_2 ==0){
                            USCITA_CANALE_2 =0;
                        }

                        break;

            //********************* E' PREMUTO IL TASTO MEMORIZZATO 3
            case TASTO_3:
                    if (JUMPER_CAN_3 ==0){
                        USCITA_CANALE_3 =1;
                    }else{
                        USCITA_CANALE_3 = !(USCITA_CANALE_3);
                    }

                    GestioneTastoPremuto();
                    if (JUMPER_CAN_3 ==0){
                        USCITA_CANALE_3 =0;
                    }
                    break;

            //********************* E' PREMUTO IL TASTO MEMORIZZATO 4
            case TASTO_4:
                    GestioneUscitaTRIAC ();
                    break;

            // Tasto non valido
            default:
                    break;
            }

            
        }

        //********RIATTIVA INTERRUPT
        FlagArrivoIR = 0;            
        INTCONbits.RBIE = 1;
        LED_ROSSO = 0;
        }

    // *********************** VERIFICA SE ANDARE IN MODALITA' SLEEP
    if ((StatoUscitaPWM == 1)||(USCITA_CANALE_2 ==1)||(USCITA_CANALE_3 ==1)||(StatoUscitaTriac ==1)){
        ContSleep = 0;
    }

    if (ContSleep > 60000){
        ContSleep = 0;
        OSCCONbits.IDLEN = 0;
        OSCCONbits.SCS0 = 1;
        OSCCONbits.SCS1 = 0;

        // disabilito timer0
        INTCONbits.TMR0IE = 0; 
        LED_GIALLO = 0;
        Sleep ();
        LED_GIALLO = 1;
        ContSleep = 1;

        // abilito timer0
        INTCONbits.TMR0IE = 1; 
    }
}
}


//**********************************************************************
//**                   APPRENDI SEQUENZA E MEMORIZZA
//**********************************************************************
void ApprendiSequenza(){

    long int TimAttesa =0;
    unsigned char NumCella ;
    unsigned char NumCellaIniziale =0;
    int FreqLampeggio = 8000;
    unsigned char MemUscitaTriac = 0;

    x2=0;
    LED_ROSSO = 1;

    while (STATO_INPUT_IR == 1){
        if (x2 >FreqLampeggio){
            x2=0;
            LED_ROSSO = !(LED_ROSSO);
        }

        TimAttesa++;
        x2++;

        // tempo per premere pulsante su telecomando IR
        if(TimAttesa > T_MAX_ATTESA){
            LED_ROSSO = 0;
        }
    }

    LED_ROSSO= 1;
    ConteggioDsi=0;
    RilevaSequenzaIR();

    if (NumVoltePremApprendi > 1){
        NumCellaIniziale = (NumVoltePremApprendi-1) * NUM_MAX_DSI ;
    }

    if (StatoUscitaTriac == 1){
        MemUscitaTriac = 1;
        StatoUscitaTriac =0;
        OUT_TRIAC = 0;
    }

    for (NumCella =0;NumCella<(NUM_MAX_DSI);NumCella++){

        write_internal_EEPROM ((NumCella+NumCellaIniziale), ContDsiRic[NumCella]);
        delay_ms (5);
    }

    if (MemUscitaTriac == 1){
        MemUscitaTriac = 0;
        StatoUscitaTriac =1;
    }

    LED_ROSSO=0;
    //return (1);
}


//**************************************************
//              RILEVA SEQUENZA IR
//**************************************************

void RilevaSequenzaIR(void){

    unsigned char DsiAtt = 0;
    unsigned char StatoInputOld;
    unsigned int ContMaxDsi =510 ;// = 255 X 2
    //unsigned int TimDsiOld =0  ;

    // il primo bit della philips  il pi corto di tutti gli altri telecomando
    FlagSegnalePhilips =0;
    StatoInputOld=STATO_INPUT_IR;
    DsiAtt = 0;

    while ((ContMaxDsi>ConteggioDsi)&& (NUM_MAX_DSI >DsiAtt)){
        if (StatoInputOld==STATO_INPUT_IR){
            ConteggioDsi ++;
        }else{
            if (DsiAtt ==0){ //Rileva la lunghezza del primo bit
                if (ConteggioDsi > PRIMO_DSI_PHILIPS){  // messo per rilevare Philiphs
                    ContMaxDsi = ConteggioDsi+1;  // non  philips
                }else{
                    FlagSegnalePhilips =1;
                }
            }

            if (ConteggioDsi < ContMaxDsi){
                if(ConteggioDsi>255){
                    ConteggioDsi= 255;
                }
                ContDsiRic[DsiAtt]=ConteggioDsi;
            }else{
                ContDsiRic[DsiAtt]=0;
            }

            ConteggioDsi=0;
            DsiAtt++ ;
            StatoInputOld=STATO_INPUT_IR;
        }

    }

    for (x2=DsiAtt;x2<NUM_MAX_DSI+1;x2++){
        ContDsiRic[x2]=0;
    }

    ConteggioDsi=0;
    DsiAtt=0;
}


//******************************************************************
//               CONTROLLA SE ARRIVA UNA SEQUENZA IR VALIDA
//*****************************************************************
unsigned char ControllaSeEunaSequenza(void){

    ConteggioDsi=0;
    //*****************CONTROLLA SE ARRIVA IL PRIMO BIT ...DEVE ESSERE LUNGO ALMENO DurataMinPrimoDsi
    while (STATO_INPUT_IR == 0){
        if(ConteggioDsi>DURATA_MIN_PRIMO_DSIi){
            RilevaSequenzaIR();
            return(1);
        }else{
            ConteggioDsi++;
        }
    }
    return(0);
}


//**********************************************************************
//**                   CONFRONTA SEQUENZA
//**********************************************************************
unsigned char ConfrontaSequenza(void){

    unsigned char Valore ;
    char ValoreMax;
    char ValoreMin;
    unsigned char DsiIniziale =1;// NON VIENE CONFRONTATO IL PRIMO Dsi
    unsigned char CellaIniziale ;
    unsigned char FlagTastoOk;

    if (FlagSegnalePhilips==1){
        DsiIniziale =5;// NON VIENE CONFRONTATO I PRIMI 5 Dsi
    }

    for (x=1;x < 5;x++){
        FlagTastoOk = 1;
        CellaIniziale = ((x-1)* NUM_MAX_DSI);

        for (x2=DsiIniziale; x2<NUM_MAX_DSI; x2++){

            Valore = read_internal_EEPROM(x2+ CellaIniziale);
            ValoreMax = Valore +DELTA;

            if(Valore > ValoreMax){
                ValoreMax =255;
            }

            ValoreMin = Valore-DELTA;

            if (ValoreMin > Valore){
                ValoreMin =0;
            }

            if((ValoreMax>=ContDsiRic[x2]) && (ContDsiRic[x2] >= ValoreMin)){
            //Risultato ok
            }else{
                FlagTastoOk=0;
                x2= 255;
            }
        }

        if (FlagTastoOk == 1){
            return(x);
        }
    }

    // Nessun tasto premuto
    return (0);
}


//************************************************************
//             CONTA QUANTE VOLTE SI PREME IL PUSANTE APPRENDI
//*************************************************************
unsigned char ContaPremiPulsante(void){

    unsigned char TimMaxPremPulsante =180;

    NumVoltePremApprendi =1;

    x2 =0;
    LED_ROSSO =1;
    delay_ms(6);// ritardo per disturbi pulsante

    while (TASTO_APPRENDI == 0){
    }

    while ((x2 < TimMaxPremPulsante )&& (NumVoltePremApprendi < 4)){
        delay_ms(6);
        LED_ROSSO = 0;
        if (TASTO_APPRENDI == 0){
            delay_ms(6);

            if (TASTO_APPRENDI == 0){
                NumVoltePremApprendi ++ ;
                x2=0;
                LED_ROSSO =1;

                while (TASTO_APPRENDI == 0){
                }
            }
        }

        x2++;
    }

    return(NumVoltePremApprendi);
}


//************************************************************
//             GESTIONE USCITA PWM   rc1  pin 12
//*************************************************************
void GestioneUscitaPWM (void){

    if(StatoUscitaPWM == 1){
        FlagRegolPwmEffettuata =0;

        FlagAumentaLuminosita = !(FlagAumentaLuminosita);
        GestioneTastoPremuto();

        if ((StatoPulsante ==0) && (FlagRegolPwmEffettuata ==0)){
            StatoUscitaPWM =0;

            // spegni gradualmente
            for( x=PercLumin; x>2; x--){ 
                SetDCPWM2 (x);
                x--;
                delay_ms(1);
            }
            SetDCPWM2 (0);
            delay_ms(100);
        }

    }else{
        StatoUscitaPWM = 1;

        // accendi gradualmente
        for( x=0;x< PercLumin;x++){  
            x++;
            if (x > PercLumin){
                x= PercLumin ;
            }

            SetDCPWM2 (x);
            delay_ms(1);
        }

        delay_ms(200);
    }

    //return (1);
}


//************************************************************
//             GESTIONE USCITA TRIAC   ra2  pin:4
//*************************************************************
void GestioneUscitaTRIAC (void){

    if(StatoUscitaTriac == 1){
        FlagRegolTriacOk =0;
        FlagAumLuminTriac = !(FlagAumLuminTriac);
        GestioneTastoPremuto();

        if ((StatoPulsante ==0) && (FlagRegolTriacOk ==0)){
            MemoriaLuminTriac = AttivaTriac ;

             // spegne gradualmente
            for( x=AttivaTriac;x< MINIMO_TRIAC;x++){
                AttivaTriac ++;
                delay_ms(5);
            }

        StatoUscitaTriac =0;
        OUT_TRIAC = 0;
        delay_ms(100);
    }

    }else{


        StatoUscitaTriac = 1;
        // accende gradualmente
        for( AttivaTriac=MINIMO_TRIAC; AttivaTriac >MemoriaLuminTriac; AttivaTriac--){
            delay_ms(5);
    }

    delay_ms(200);
    }
}


//**********************************************************************
//             REGOLA LUMINOSITA PWM                
//**********************************************************************
void RegolaLuminositaPWM (void){

    if (FlagAumentaLuminosita == 0){
        PercLumin = PercLumin-50;
    }else{
        PercLumin = PercLumin + 50;

        if (PercLumin > 1023){
            PercLumin = 1023;
            FlagAumentaLuminosita = 0;
        }
    }

    SetDCPWM2 (PercLumin);
    delay_ms(2);

    if (PercLumin < 50) {
       FlagAumentaLuminosita = 1;
    }

}


//**********************************************************************
//             REGOLA LUMINOSITA TRIAC                 
//**********************************************************************
void RegolaLuminositaTriac (void){
    unsigned char Variazione;

    Variazione = (MINIMO_TRIAC - AttivaTriac) /10;
    if (Variazione == 0){
        Variazione = 1;
    }

    if (FlagAumLuminTriac == 0){
        AttivaTriac = AttivaTriac + Variazione;
    }else{
        AttivaTriac = AttivaTriac - Variazione;
    }

    //  arrivato al minimo luminosita...inverte
    if((AttivaTriac > MINIMO_TRIAC)&& (AttivaTriac< (MINIMO_TRIAC +10))){
        AttivaTriac = MINIMO_TRIAC;
        FlagAumLuminTriac =1;
    }

    //  arrivato al massimo della luminosit...inverte!
    if (AttivaTriac > 200){
        AttivaTriac = 0;
        FlagAumLuminTriac =0;
    }
}


//**********************************************************************
//           GESTIONE TASTO MANTENUTO PREMUTO                
//**********************************************************************
void GestioneTastoPremuto (void){

    StatoPulsante =1;

    while (StatoPulsante == 1){
        // Attende tempo per rilascio tasto
        delay_ms(50);
        // Controlla se il tasto  rilasciato o se si tratta dell'intervallo di sequenza
        x=0;

        while ((STATO_INPUT_IR == 1) && (x < TimMaxIntervalloImpulsiIR)){
            x++;
        }

        if (x <TimMaxIntervalloImpulsiIR){

            if(ControllaSeEunaSequenza()){
                StatoPulsante =1;

                switch (NumTastoPremuto){
                    case 1://********************* E' PREMUTO UN TASTO MEMORIZZATO 1
                            RegolaLuminositaPWM();
                            FlagRegolPwmEffettuata =1;
                            break;
                    case 4://********************* E' PREMUTO UN TASTO MEMORIZZATO 4
                            RegolaLuminositaTriac();
                            FlagRegolTriacOk =1;
                            break;
                }

            }
        }else{
             StatoPulsante =0;
             x=x;
        }
    }
}


//**********************************************************************
//       TEST USCITE               
//**********************************************************************
void TestUscite (void){

    // *************************************  test uscita PWM
    StatoUscitaPWM = 1;

    for( x=0;x< PercLumin;x++){  // accendi gradualmente
        x++;
        if (x > PercLumin){
            x= PercLumin ;
        }

        SetDCPWM2 (x);
        delay_ms(1);
    }

    delay_ms(500);

    for( x=PercLumin;x>2;x--){ // spegni gradualmente
        SetDCPWM2 (x);
        x--;
        delay_ms(1);
    }

    SetDCPWM2 (0);
    StatoUscitaPWM = 0;
    delay_ms(200);
    // *************************************test uscita canale 1
    USCITA_CANALE_2 = 1;
    delay_ms(500);
    USCITA_CANALE_2 = 0;
    delay_ms(200);
    // **************************************test uscita canale 3
    USCITA_CANALE_3 = 1;
    delay_ms(500);
    USCITA_CANALE_3 = 0;
    // ***************************************test uscita triac
    StatoUscitaTriac = 1;

    for( AttivaTriac=MINIMO_TRIAC;AttivaTriac >30;AttivaTriac--){ // accende gradualmente
        delay_ms(5);
    }

    delay_ms(600);

    for( x=AttivaTriac;x< MINIMO_TRIAC;x++){ // spegne gradualmente
        AttivaTriac ++;
        delay_ms(5);
    }
    
    StatoUscitaTriac =0;
    delay_ms(100);
}
